# -*- coding: utf-8 -*-
import re

LATIN_LIKE_SCRIPTS = ['Latin', 'Common', 'Cyrillic', 'Greek', 'Hebrew']

def readScriptRanges(scripts=None):
    """
    Read script ranges from http://unicode.org/Public/UNIDATA/Scripts.txt file.
    """
    scripts = scripts or LATIN_LIKE_SCRIPTS
    ranges = []

    f = open('Scripts.txt', 'r')
    for line in f:
        line = line.strip('\n')
        matchObj = re.match(
            '^([0123456789ABCDEF]{4}(\.\.[0123456789ABCDEF]{4})?)\s*;\s+(%s)\s+(#.*)?$'
                % '|'.join(scripts),
            line)
        if matchObj:
            entry = matchObj.group(1)
            if len(entry) > 4:
                start, stop = entry.split('..', 1)
                ranges.append((start, stop))
            else:
                ranges.append(entry)
    f.close()

    return ranges

class CharacterRangeIterator(object):
    """Iterates over a given set of codepoint ranges given in hex."""
    def __init__(self, ranges):
        self.ranges = ranges[:]
        self._curRange = self._popRange()
    def _popRange(self):
        if self.ranges:
            charRange = self.ranges[0]
            del self.ranges[0]
            if type(charRange) == type(()):
                rangeFrom, rangeTo = charRange
            else:
                rangeFrom, rangeTo = (charRange, charRange)
            return (int(rangeFrom, 16), int(rangeTo, 16))
        else:
            return []
    def __iter__(self):
        return self
    def __next__(self):
        if not self._curRange:
            raise StopIteration

        curIndex, toIndex = self._curRange
        if curIndex < toIndex:
            self._curRange = (curIndex + 1, toIndex)
        else:
            self._curRange = self._popRange()
        return chr(curIndex)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Simple module that "flips" latin characters in a string to create an
"upside-down" impression. Makes extensive use of compatible latin characters
encoded in Unicode.

Support for diacritics offered through combining diacritical marks. Depends on
proper rendering though.

2008-2010 Christoph Burgmer (cburgmer@ira.uka.de)

            ˙ƎᴚⱯM⊥ᖵOS ƎH⊥ NI S⅁NIꞀⱯƎᗡ ᴚƎH⊥O ᴚO ƎS∩ ƎH⊥ ᴚO ƎᴚⱯM⊥ᖵOS ƎH⊥ H⊥IM
       NOI⊥ƆƎNNOƆ NI ᴚO ᖵO ⊥∩O 'WOᴚᖵ ⅁NISIᴚⱯ 'ƎSIMᴚƎH⊥O ᴚO ⊥ᴚO⊥ '⊥ƆⱯᴚ⊥NOƆ ᖵO
        NOI⊥ƆⱯ NⱯ NI ᴚƎH⊥ƎHM '⅄⊥IꞀIᗺⱯIꞀ ᴚƎH⊥O ᴚO SƎ⅁ⱯWⱯᗡ 'WIⱯꞀƆ ⅄NⱯ ᴚOᖵ ƎꞀᗺⱯIꞀ
       Ǝᗺ SᴚƎᗡꞀOH ⊥H⅁Iᴚ⅄ԀOƆ ᴚO SᴚOH⊥∩Ɐ ƎH⊥ ꞀꞀⱯHS ⊥NƎɅƎ ON NI ˙⊥NƎWƎ⅁NIᴚᖵNINON
                         ᗡNⱯ ƎSOԀᴚ∩Ԁ ᴚⱯꞀ∩ƆI⊥ᴚⱯԀ Ɐ ᴚOᖵ SSƎN⊥Iᖵ '⅄⊥IꞀIᗺⱯ⊥NⱯHƆᴚƎW
          ᖵO SƎI⊥NⱯᴚᴚⱯM ƎH⊥ O⊥ ᗡƎ⊥IWIꞀ ⊥ON ⊥∩ᗺ ⅁NIᗡ∩ꞀƆNI 'ᗡƎIꞀԀWI ᴚO SSƎᴚԀXƎ
             'ᗡNI⋊ ⅄NⱯ ᖵO ⅄⊥NⱯᴚᴚⱯM ⊥∩OH⊥IM '„SI SⱯ„ ᗡƎᗡIɅOᴚԀ SI ƎᴚⱯM⊥ᖵOS ƎH⊥

                   ˙ǝɹɐʍʇɟoS ǝɥʇ ɟo suoᴉʇɹod ꞁɐᴉʇuɐʇsqns ɹo sǝᴉdoɔ ꞁꞁɐ uᴉ pǝpnꞁɔuᴉ
                  ǝq ꞁꞁɐɥs ǝɔᴉʇou uoᴉssᴉɯɹǝd sᴉɥʇ puɐ ǝɔᴉʇou ʇɥƃᴉɹʎdoɔ ǝʌoqɐ ǝɥ⊥

                                                        :suoᴉʇᴉpuoɔ ƃuᴉʍoꞁꞁoɟ ǝɥʇ
           oʇ ʇɔǝɾqns 'os op oʇ pǝɥsᴉuɹnɟ sᴉ ǝɹɐʍʇɟoS ǝɥʇ ɯoɥʍ oʇ suosɹǝd ʇᴉɯɹǝd
                oʇ puɐ 'ǝɹɐʍʇɟoS ǝɥʇ ɟo sǝᴉdoɔ ꞁꞁǝs ɹo/puɐ 'ǝsuǝɔᴉꞁqns 'ǝʇnqᴉɹʇsᴉp
              'ɥsᴉꞁqnd 'ǝƃɹǝɯ 'ʎɟᴉpoɯ 'ʎdoɔ 'ǝsn oʇ sʇɥƃᴉɹ ǝɥʇ uoᴉʇɐʇᴉɯᴉꞁ ʇnoɥʇᴉʍ
              ƃuᴉpnꞁɔuᴉ 'uoᴉʇɔᴉɹʇsǝɹ ʇnoɥʇᴉʍ ǝɹɐʍʇɟoS ǝɥʇ uᴉ ꞁɐǝp oʇ '(„ǝɹɐʍʇɟoS„
                 ǝɥʇ) sǝꞁᴉɟ uoᴉʇɐʇuǝɯnɔop pǝʇɐᴉɔossɐ puɐ ǝɹɐʍʇɟos sᴉɥʇ ɟo ʎdoɔ ɐ
           ƃuᴉuᴉɐʇqo uosɹǝd ʎuɐ oʇ 'ǝƃɹɐɥɔ ɟo ǝǝɹɟ 'pǝʇuɐɹƃ ʎqǝɹǝɥ sᴉ uoᴉssᴉɯɹǝԀ

                                                                   ⊥IW :ǝsuǝɔᴉꞀ
"""

from __future__ import unicode_literals

__all__ = ["transform"]
__version__ = '0.4'
__author__ = 'Christoph Burgmer <christoph.burgmer@gmail.com>'
__url__ = 'http://github.com/cburgmer/upsidedown'
__license__ = 'MIT'

import unicodedata
import string

# Define dual character. Make sure that mapping is bijective.
FLIP_RANGES = [
    (string.ascii_lowercase, "ɐqɔpǝɟƃɥᴉɾʞꞁɯuodbɹsʇnʌʍxʎz"),
    # alternatives: l:ʅ
    (string.ascii_uppercase, "ⱯᗺƆᗡƎᖵ⅁HIᒋ⋊ꞀWNOԀꝹᴚS⊥∩ɅMX⅄Z"),
    # alternatives: L:ᒣ⅂, J:ſ, F:߃Ⅎ, A:∀ᗄ, U:Ⴖ, W:Ϻ, C:ϽↃ, Q:Ό, M:Ɯꟽ
    (string.digits, "0ІᘔƐᔭ59Ɫ86"),
    (string.punctuation, "¡„#$%⅋,)(*+'-˙/:؛>=<¿@]\\[ᵥ‾`}|{~"),
    ]
# See also http://www.fileformat.info/convert/text/upside-down-map.htm

# See:
# http://de.wikipedia.org/wiki/Unicode-Block_Kombinierende_diakritische_Zeichen
UNICODE_COMBINING_DIACRITICS = {'̈': '̤', '̊': '̥', '́': '̗', '̀': '̖',
    '̇': '̣', '̃': '̰', '̄': '̱', '̂': '̬', '̆': '̯', '̌': '̭',
    '̑': '̮', '̍': '̩'}

TRANSLITERATIONS = {'ß': 'ss'}

# character lookup
_CHARLOOKUP = {}
for chars, flipped in FLIP_RANGES:
    _CHARLOOKUP.update(zip(chars, flipped))

# get reverse direction
for char in _CHARLOOKUP.copy():
    # make 1:1 back transformation possible
    assert (_CHARLOOKUP[char] not in _CHARLOOKUP
        or _CHARLOOKUP[_CHARLOOKUP[char]] == char), \
        ("%s has ambiguous mapping" % _CHARLOOKUP[char])
    _CHARLOOKUP[_CHARLOOKUP[char]] = char

# lookup for diacritical marks, reverse first
_DIACRITICSLOOKUP = dict([(UNICODE_COMBINING_DIACRITICS[char], char)
    for char in UNICODE_COMBINING_DIACRITICS])
_DIACRITICSLOOKUP.update(UNICODE_COMBINING_DIACRITICS)

def transform(string, transliterations=None):
    """
    Transform the string to "upside-down" writing.

    Example:

        >>> import upsidedown
        >>> print(upsidedown.transform('Hello World!'))
        ¡pꞁɹoM oꞁꞁǝH

    For languages with diacritics you might want to supply a transliteration to
    work around missing (rendering of) upside-down forms:
        >>> import upsidedown
        >>> print(upsidedown.transform('köln', transliterations={'ö': 'oe'}))
        uꞁǝoʞ
    """
    transliterations = transliterations or TRANSLITERATIONS

    for character in transliterations:
        string = string.replace(character, transliterations[character])

    inputChars = list(string)
    inputChars.reverse()

    output = []
    for character in inputChars:
        if character in _CHARLOOKUP:
            output.append(_CHARLOOKUP[character])
        else:
            charNormalised = unicodedata.normalize("NFD", character)

            for c in charNormalised[:]:
                if c in _CHARLOOKUP:
                    charNormalised = charNormalised.replace(c, _CHARLOOKUP[c])
                elif c in _DIACRITICSLOOKUP:
                    charNormalised = charNormalised.replace(c,
                        _DIACRITICSLOOKUP[c])

            output.append(unicodedata.normalize("NFC", charNormalised))

    return ''.join(output)

def main():
    """Main method for running upsidedown.py from the command line."""
    import sys

    output = []
    line = sys.stdin.readline()

    while line:
        line = line.strip("\n")
        output.append(transform(line))

        line = sys.stdin.readline()
    output.reverse()
    print("\n".join(output))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Find a character forms for a given character in a range of Unicode scripts.

Example::
    $ python3 findchar.py Q
    'ℚ' ℚ DOUBLE-STRUCK CAPITAL Q
    '℺' ℺ ROTATED CAPITAL Q
    '⒬' ⒬ PARENTHESIZED LATIN SMALL LETTER Q
    'Ⓠ' Ⓠ CIRCLED LATIN CAPITAL LETTER Q
    'ⓠ' ⓠ CIRCLED LATIN SMALL LETTER Q
    ...

Needs Python3.

You need the Scripts.txt file from Unicode::
    $ wget http://unicode.org/Public/UNIDATA/Scripts.txt
"""

import unicodedata
import re
import sys
from util import readScriptRanges, CharacterRangeIterator

char = sys.argv[1]

ranges = readScriptRanges()
for c in CharacterRangeIterator(ranges):
    try:
        name = unicodedata.name(c)
    except ValueError:
        continue
    if re.match('(%(c)s$|%(c)s |.* %(c)s$|.* %(c)s )' % {'c': char}, name):
        print(repr(c), c, name)



import re
import codecs

from setuptools import setup
import upsidedown

VERSION = str(upsidedown.__version__)
(AUTHOR, EMAIL) = re.match('^(.*?)\s*<(.*)>$', upsidedown.__author__).groups()
URL = upsidedown.__url__
LICENSE = upsidedown.__license__


with codecs.open('README.rst', encoding='utf-8') as readme:
    long_description = readme.read()

setup(name='upsidedown',
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description='"Flip" characters in a string to create an "upside-down" impression.',
    long_description=long_description,
    url=URL,
    download_url='http://github.com/cburgmer/upsidedown/downloads',
    py_modules=['upsidedown'],
    test_suite='tests',
    entry_points={
        'console_scripts': [
            'upsidedown = upsidedown:main',
        ],
    },
    license=LICENSE,
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Topic :: Text Processing',
        ])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Find all characters in Latin like scripts that are flagged REVERSED or TURNED.

Example::
    $ python3 checkrange.py
    'ʻ' ʻ MODIFIER LETTER TURNED COMMA
    'ʽ' ʽ MODIFIER LETTER REVERSED COMMA
    'ˁ' ˁ MODIFIER LETTER REVERSED GLOTTAL STOP
    '‛' ‛ SINGLE HIGH-REVERSED-9 QUOTATION MARK
    '‟' ‟ DOUBLE HIGH-REVERSED-9 QUOTATION MARK
    '‵' ‵ REVERSED PRIME
    ...

Needs Python3.

You need the Scripts.txt file from Unicode::
    $ wget http://unicode.org/Public/UNIDATA/Scripts.txt
"""
import unicodedata
from util import readScriptRanges, CharacterRangeIterator

ranges = readScriptRanges()
for c in CharacterRangeIterator(ranges):
    try:
        name = unicodedata.name(c)
    except ValueError:
        continue
    if 'REVERSED' in name or 'TURNED' in name:
        print(repr(c), c, name)
